package org.processmining.plugins.petrinet.importing.pnml;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.processmining.contexts.gui.GUIPluginContext;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.graphbased.directed.petrinet.InhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.ResetInhibitorNet;
import org.processmining.models.graphbased.directed.petrinet.ResetNet;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.InhibitorArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.ResetArc;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.semantics.petrinet.Marking;

/**
 * Exports a net (Reset/Inhibitor net, Reset net, Inhibitor net, or a Petri net)
 * to a PNML file.
 * 
 * @author hverbeek
 * 
 */
@Plugin(name = "PNML export", returnLabels = {}, returnTypes = {}, parameterLabels = { "Petri net", "File" }, userAccessible = true)
//@UIExportPlugin(description = "PNML file", extension = "pnml")
public class PnmlExport {

	/**
	 * Exports a Reset/Inhibitor net to a PNML file.
	 * 
	 * @param context
	 *            The current GUI context.
	 * @param riNet
	 *            The Reset/Inhibitor net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Reset/Inhibitor net)", requiredParameterLabels = { 0 })
	public void exportResetInhibitorNet(GUIPluginContext context, ResetInhibitorNet riNet) throws IOException {
		export(context, riNet, riNet.getLabel(), riNet.getPlaces(), riNet.getTransitions(), riNet.getEdges(), context
				.saveFile("PNML"));
	}

	/**
	 * Exports a Reset net to a PNML file.
	 * 
	 * @param context
	 *            The current GUI context.
	 * @param rNet
	 *            The Reset net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Reset net)", requiredParameterLabels = { 0 })
	public void exportResetNet(GUIPluginContext context, ResetNet rNet) throws IOException {
		export(context, rNet, rNet.getLabel(), rNet.getPlaces(), rNet.getTransitions(), rNet.getEdges(), context
				.saveFile("PNML"));
	}

	/**
	 * Exports an Inhibitor net to a PNML file.
	 * 
	 * @param context
	 *            The current GUI context.
	 * @param rNet
	 *            The Inhibitor net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Inhibitor net)", requiredParameterLabels = { 0 })
	public void exportInhibitorNet(GUIPluginContext context, InhibitorNet iNet) throws IOException {
		export(context, iNet, iNet.getLabel(), iNet.getPlaces(), iNet.getTransitions(), iNet.getEdges(), context
				.saveFile("PNML"));
	}

	/**
	 * Exports a Petri net to a PNML file.
	 * 
	 * @param context
	 *            The current GUI context.
	 * @param rNet
	 *            The Petri net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Petri net)", requiredParameterLabels = { 0 })
	public void exportPetriNet(GUIPluginContext context, Petrinet pNet) throws IOException {
		export(context, pNet, pNet.getLabel(), pNet.getPlaces(), pNet.getTransitions(), pNet.getEdges(), context
				.saveFile("PNML"));
	}

	/**
	 * Exports a Reset/Inhibitor net to a PNML file.
	 * 
	 * @param context
	 *            The current context.
	 * @param riNet
	 *            The Reset/Inhibitor net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Reset/Inhibitor net)", requiredParameterLabels = { 0, 1 })
	public void exportResetInhibitorNet(PluginContext context, ResetInhibitorNet riNet, File file) throws IOException {
		export(context, riNet, riNet.getLabel(), riNet.getPlaces(), riNet.getTransitions(), riNet.getEdges(), file);
	}

	/**
	 * Exports a Reset net to a PNML file.
	 * 
	 * @param context
	 *            The current context.
	 * @param rNet
	 *            The Reset net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Reset net)", requiredParameterLabels = { 0, 1 })
	public void exportResetNet(PluginContext context, ResetNet rNet, File file) throws IOException {
		export(context, rNet, rNet.getLabel(), rNet.getPlaces(), rNet.getTransitions(), rNet.getEdges(), file);
	}

	/**
	 * Exports an Inhibitor net to a PNML file.
	 * 
	 * @param context
	 *            The current context.
	 * @param rNet
	 *            The Inhibitor net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Inhibitor net)", requiredParameterLabels = { 0, 1 })
	public void exportInhibitorNet(PluginContext context, InhibitorNet iNet, File file) throws IOException {
		export(context, iNet, iNet.getLabel(), iNet.getPlaces(), iNet.getTransitions(), iNet.getEdges(), file);
	}

	/**
	 * Exports a Petri net to a PNML file.
	 * 
	 * @param context
	 *            The current context.
	 * @param rNet
	 *            The Petri net to export.
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	@PluginVariant(variantLabel = "PNML export (Petri net)", requiredParameterLabels = { 0, 1 })
	public void exportPetriNet(PluginContext context, Petrinet pNet, File file) throws IOException {
		export(context, pNet, pNet.getLabel(), pNet.getPlaces(), pNet.getTransitions(), pNet.getEdges(), file);
	}

	/**
	 * Exports a net (that is, a collection of places, a collection of
	 * transitions, and a collection of edges) to a PNML file.
	 * 
	 * @param context
	 *            The current context.
	 * @param label
	 *            The label of the net.
	 * @param places
	 *            The collection of places.
	 * @param transitions
	 *            The collection of transitions.
	 * @param arcs
	 *            The collection of arcs (may include reset and inhibitor arcs).
	 * @return The absolute path to the exported PNML file.
	 * @throws IOException
	 */
	private void export(PluginContext context, PetrinetGraph net, String label, Collection<? extends Place> places,
			Collection<? extends Transition> transitions,
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> arcs, File file)
			throws IOException {
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file)));
		Marking marking;
//		try {
//			marking = context.tryToFindOrConstructFirstObject(Marking.class, MarkingNetConnection.class,
//					MarkingNetConnection.MARKING, net);
//		} catch (ConnectionCannotBeObtained e) {
			// use empty marking\
			marking = new Marking();
//		}
		bw.write(toPNML(label, places, transitions, arcs, marking));
		bw.close();
	}

	/**
	 * Creates a PNML-formatted string from the given net.
	 * 
	 * @param label
	 *            The label of the net.
	 * @param places
	 *            The collection of places.
	 * @param transitions
	 *            The collection of transitions.
	 * @param arcs
	 *            The collection of arcs (may include reset and inhibitor arcs).
	 * @return A PNML representation of the net.
	 */
	private String toPNML(String label, Collection<? extends Place> places,
			Collection<? extends Transition> transitions,
			Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> arcs, Marking marking) {
		StringBuffer sb = new StringBuffer();
		/**
		 * Used as global id for the objects in the PNML representation.
		 */
		int id = 1;
		/**
		 * Maps nodes (places, transitions) to their ids. Used for arcs.
		 */
		Map<PetrinetNode, String> idMap = new HashMap<PetrinetNode, String>();
		/**
		 * Prologue.
		 */
		sb.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
		sb.append(PnmlTags.PNML.getStart() + "\n");
		sb.append("  "
				+ PnmlTags.NET.getStart(PnmlAttributes.ID.getAssign("n" + id++)
						+ PnmlAttributes.TYPE.getAssign("http://www.yasper.org/specs/epnml-1.1")) + "\n");
		sb.append("    <" + PnmlTags.NAME.getLabel() + "><" + PnmlTags.TEXT.getLabel() + ">"
				+ XmlEntities.encode(label) + "</" + PnmlTags.TEXT.getLabel() + "></" + PnmlTags.NAME.getLabel()
				+ ">\n");
		/**
		 * Places.
		 */
		for (Place place : places) {
			idMap.put(place, "p" + id);
			sb.append("    <" + PnmlTags.PLACE.getLabel() + " " + PnmlAttributes.ID.getLabel() + "=\"p" + id++
					+ "\">\n");
			if (!place.getId().toString().equals(place.getLabel())) {
				sb.append("      <" + PnmlTags.NAME.getLabel() + "><" + PnmlTags.TEXT.getLabel() + ">"
						+ XmlEntities.encode(place.getLabel()) + "</" + PnmlTags.TEXT.getLabel() + "></"
						+ PnmlTags.NAME.getLabel() + ">\n");
			}
			if (marking.occurrences(place) > 0) {
				sb.append("      <" + PnmlTags.INITIALMARKING.getLabel() + "><" + PnmlTags.TEXT.getLabel() + ">"
						+ marking.occurrences(place) + "</" + PnmlTags.TEXT.getLabel() + "></"
						+ PnmlTags.INITIALMARKING.getLabel() + ">\n");
			}
			sb.append("    </" + PnmlTags.PLACE.getLabel() + ">\n");
		}
		/**
		 * Transitions.
		 */
		for (Transition transition : transitions) {
			idMap.put(transition, "t" + id);
			sb.append("    <" + PnmlTags.TRANSITION.getLabel() + " " + PnmlAttributes.ID.getLabel() + "=\"t" + id++
					+ "\">\n");
			if (!transition.isInvisible() || !transition.getId().toString().equals(transition.getLabel())) {
				sb.append("      <" + PnmlTags.NAME.getLabel() + "><" + PnmlTags.TEXT.getLabel() + ">"
						+ XmlEntities.encode(transition.getLabel()) + "</" + PnmlTags.TEXT.getLabel() + "></"
						+ PnmlTags.NAME.getLabel() + ">\n");
			}
			sb.append("    </" + PnmlTags.TRANSITION.getLabel() + ">\n");
		}
		/**
		 * Arcs.
		 */
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> arc : arcs) {
			PetrinetNode source = arc.getSource();
			PetrinetNode target = arc.getTarget();
			if (idMap.containsKey(source) && idMap.containsKey(target)) {
				sb.append("    <" + PnmlTags.ARC.getLabel() + " " + PnmlAttributes.TARGET.getLabel() + "=\""
						+ idMap.get(target) + "\" " + PnmlAttributes.SOURCE.getLabel() + "=\"" + idMap.get(source)
						+ "\" " + PnmlAttributes.ID.getLabel() + "=\"a" + id++ + "\">\n");
				if (InhibitorArc.class.isAssignableFrom(arc.getClass())) {
					/**
					 * Inhibitor arc.
					 */
					sb.append("      <" + PnmlTags.TYPE.getLabel() + "><" + PnmlTags.TEXT.getLabel() + ">"
							+ PnmlTexts.INHIBITOR.getLabel() + "</" + PnmlTags.TEXT.getLabel() + "></"
							+ PnmlTags.TYPE.getLabel() + ">\n");
				} else if (ResetArc.class.isAssignableFrom(arc.getClass())) {
					/**
					 * Reset arc.
					 */
					sb.append("      <" + PnmlTags.TYPE.getLabel() + "><" + PnmlTags.TEXT.getLabel() + ">"
							+ PnmlTexts.RESET.getLabel() + "</" + PnmlTags.TEXT.getLabel() + "></"
							+ PnmlTags.TYPE.getLabel() + ">\n");
				} else {
					/**
					 * Regular arc, may have a non-default (1) weight.
					 */
					Arc myArc = (Arc) arc;
					if (myArc.getWeight() != 1) {
						sb.append("      <" + PnmlTags.INSCRIPTION.getLabel() + "><" + PnmlTags.TEXT.getLabel() + ">"
								+ myArc.getWeight() + "</" + PnmlTags.TEXT.getLabel() + "></"
								+ PnmlTags.INSCRIPTION.getLabel() + ">\n");
					}
				}
				sb.append("    </" + PnmlTags.ARC.getLabel() + ">\n");
			}
		}
		/**
		 * Epilogue.
		 */
		sb.append("  </" + PnmlTags.NET.getLabel() + ">\n");
		sb.append("</" + PnmlTags.PNML.getLabel() + ">\n");
		return sb.toString();
	}
}
