package org.processmining.plugins.testbed.generationrule;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;


/**
 * the class implements the Place Refinement rule 
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 * 
 */
public class PlaceRefinement extends GenerationRule
{
	private Place toBeRefined;

	
	public PlaceRefinement(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		this.toBeRefined = null;
		
		selectElement(); 
		
		if (!notFound)
		{
			addElementToPnet();
			removeElement();
		}
	}
	

	/**
	 * selects a place to apply the rule
	 */
	protected void selectElement()
	{
		int weight1 = 0;
		int weight2 = 0;
		boolean changed1 = false;
		boolean changed2 = false;
	
		
		if (initialNodeOfNet != null && initialNodeOfNet instanceof Place)
		{
			weight1 = elementWeight.getElementWeight((Place)initialNodeOfNet);
			elementWeight.changeElementWeight((Place)initialNodeOfNet, 0);
			changed1 = true;
		}
		
		if (finalNodeOfNet != null && finalNodeOfNet instanceof Place)
		{
			weight2 = elementWeight.getElementWeight((Place)finalNodeOfNet);
			elementWeight.changeElementWeight((Place)finalNodeOfNet, 0);
			changed2 = true;
		}
		
		int totalWeight = elementWeight.getPlaceTotalWeight();
		
		if (totalWeight == 0)
		{
			notFound = true;
			
			if (changed1)
			{
				elementWeight.changeElementWeight((Place)initialNodeOfNet, weight1);
			}
			
			if (changed2)
			{
				elementWeight.changeElementWeight((Place)finalNodeOfNet, weight2);
			}
			
			return;
		}
			
		Map<Place, Integer> placeMap = elementWeight.getPlaceWeight();//get weights of all the transitions in the net
		int random = new Random().nextInt(totalWeight);
		int weightSoFar = 0;
		Iterator<Place> it = placeMap.keySet().iterator();//to iterate all transitions in the key set

		while(it.hasNext())
		{
			Place p = it.next();
			weightSoFar += placeMap.get(p);
			
			if (weightSoFar > random)
			{
				toBeRefined = p;
				break;
			}
		}
		
		if (changed1)
		{
			elementWeight.changeElementWeight((Place)initialNodeOfNet, weight1);
		}
		
		if (changed2)
		{
			elementWeight.changeElementWeight((Place)finalNodeOfNet, weight2);
		}
		
//		if (toBeRefined == null)
//		{
//			notFound = true;
//		}
	}
	

	/**
	 * adds a place and a transition and arcs to the net 
	 */
	protected void addElementToPnet()
	{
		Transition t = null;

		Place p2 = null;
		Collection<Transition> sourceTransitions = new Vector<Transition>();
		Collection<Transition> targetTransitions = new Vector<Transition>();
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getInEdges(toBeRefined))
		{
			sourceTransitions.add((Transition)e.getSource());
		}
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getOutEdges(toBeRefined))
		{
			targetTransitions.add((Transition)e.getTarget());
		}
		
		Place p1 = pnet.addPlace(toBeRefined.getLabel());
		elementWeight.setWeight(p1, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		for (Transition sourceTrans : sourceTransitions)
		{
			Arc arc = pnet.addArc(sourceTrans, p1);
			elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		}
		
		t = pnet.addTransition("t" + (pnet.getTransitions().size()));
		elementWeight.setWeight(t, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		Arc arc = pnet.addArc(p1, t);
		elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		p2 = pnet.addPlace("p" + (pnet.getPlaces().size() - 1));
		elementWeight.setWeight(p2, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		Arc arc1 = pnet.addArc(t, p2);
		elementWeight.setWeight(arc1, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		for (Transition targetTrans : targetTransitions)
		{
			Arc arc3 = pnet.addArc(p2, targetTrans);
			elementWeight.setWeight(arc3, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		}
	}
	
	private void removeElement()
	{
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> sourceEdges = pnet.getInEdges(toBeRefined);
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> targetEdges = pnet.getOutEdges(toBeRefined);
		Iterator<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> sourceIter = sourceEdges.iterator();
		Iterator<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> targetIter = targetEdges.iterator();
	
		while (sourceIter.hasNext())
		{
			Arc arc = (Arc) sourceIter.next();
			elementWeight.removeElementWeight(arc);
			pnet.removeEdge(arc);//sourceIter.next());
			
			sourceEdges = pnet.getInEdges(toBeRefined);
			sourceIter = sourceEdges.iterator();
		}
	
		while (targetIter.hasNext())
		{
			Arc arc = (Arc) targetIter.next();
			elementWeight.removeElementWeight(arc);
			pnet.removeEdge(arc);//targetIter.next());
			
			targetEdges = pnet.getOutEdges(toBeRefined);
			targetIter = targetEdges.iterator();
		}

		elementWeight.removeElementWeight(toBeRefined);
		pnet.removeNode(toBeRefined);
	}
}

