package org.processmining.plugins.testbed.generationrule;

import java.util.Collection;
import java.util.Iterator;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import org.processmining.models.graphbased.directed.petrinet.Petrinet;
import org.processmining.models.graphbased.directed.petrinet.PetrinetEdge;
import org.processmining.models.graphbased.directed.petrinet.PetrinetNode;
import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.plugins.testbed.util.ElementWeight;


/**
 * the class implements the Transition Refinement rule
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 * 
 */
public class TransitionRefinement extends GenerationRule
{
	private Transition toBeRefined;

	
	public TransitionRefinement(Petrinet net, PetrinetNode initialNodeOfNet, PetrinetNode finalNodeOfNet, ElementWeight elementWeight)
	{
		super(net, initialNodeOfNet, finalNodeOfNet, elementWeight);
		toBeRefined = null;
		
		selectElement(); 
		
		if (!notFound)
		{
			addElementToPnet();
			removeElement();
		}
	}
	

	
	/**
	 * selects a transition to apply the rule
	 */
	protected void selectElement()
	{
		int weight1 = 0;
		int weight2 = 0;
		boolean changed1 = false;
		boolean changed2 = false;
		
		if (initialNodeOfNet != null && initialNodeOfNet instanceof Transition)
		{
			weight1 = elementWeight.getElementWeight((Transition)initialNodeOfNet);
			elementWeight.changeElementWeight((Transition)initialNodeOfNet, 0);
			changed1 = true;
		}
		
		if (finalNodeOfNet != null && finalNodeOfNet instanceof Transition)
		{
			weight2 = elementWeight.getElementWeight((Transition)finalNodeOfNet);
			elementWeight.changeElementWeight((Transition)finalNodeOfNet, 0);
			changed2 = true;
		}		
		
		int totalWeight = elementWeight.getTransitionTotalWeight();		
		
		if (totalWeight == 0)
		{
			notFound = true;
			
			if (changed1)
			{
				elementWeight.changeElementWeight((Transition)initialNodeOfNet, weight1);
			}
			
			if (changed2)
			{
				elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight2);
			}
			
			return;			
		}
		
		Map<Transition, Integer> transMap = elementWeight.getTransitionWeight();
		int random = new Random().nextInt(totalWeight);
		int weightSoFar = 0;
		Iterator<Transition> it = transMap.keySet().iterator();

		while(it.hasNext())
		{
			Transition t = it.next();
			weightSoFar += transMap.get(t);
			
			if (weightSoFar > random)
			{
				toBeRefined = t;
				break;
			}
		}
		
		if (changed1)
		{
			elementWeight.changeElementWeight((Transition)initialNodeOfNet, weight1);
		}
		
		if (changed2)
		{
			elementWeight.changeElementWeight((Transition)finalNodeOfNet, weight2);
		}
		
//		if (toBeRefined == null)
//		{
//			notFound = true;
//		}
	}

	
	/**
	 * adds a place and a transition and arcs to the net
	 */
	protected void addElementToPnet()
	{
		Collection<Place> inputPlaces = new Vector<Place>();
		Collection<Place> outputPlaces = new Vector<Place>();
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getInEdges(toBeRefined))
		{
			inputPlaces.add((Place)e.getSource());
		}
		
		for (PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode> e : pnet.getOutEdges(toBeRefined))
		{
			outputPlaces.add((Place)e.getTarget());
		}
		
		Transition t1 = pnet.addTransition(toBeRefined.getLabel());
		elementWeight.setWeight(t1, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		for (Place sourcePlace : inputPlaces)
		{
			Arc arc = pnet.addArc(sourcePlace, t1);
			elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		}
		
		Place p = pnet.addPlace("p" + (pnet.getPlaces().size()));
		elementWeight.setWeight(p, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		Arc arc1 = pnet.addArc(t1,p);
		elementWeight.setWeight(arc1, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		Transition t2 = pnet.addTransition("t" + (pnet.getTransitions().size() - 1));
		elementWeight.setWeight(t2, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
	
		Arc arc2 = pnet.addArc(p, t2);
		elementWeight.setWeight(arc2, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		
		for (Place targetPlace : outputPlaces)
		{
			Arc arc = pnet.addArc(t2, targetPlace);
			elementWeight.setWeight(arc, elementWeight.addElementWeight(elementWeight.getElementWeight(toBeRefined)));
		}	
	}
	

	private void removeElement()
	{
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> sourceEdges = pnet.getInEdges(toBeRefined);
		Collection<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> targetEdges = pnet.getOutEdges(toBeRefined);
		Iterator<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> sourceIter = sourceEdges.iterator();
		Iterator<PetrinetEdge<? extends PetrinetNode, ? extends PetrinetNode>> targetIter = targetEdges.iterator();

		while (sourceIter.hasNext())
		{
			Arc arc = (Arc) sourceIter.next();
			elementWeight.removeElementWeight(arc);
			pnet.removeEdge(arc);//sourceIter.next());
			
			sourceEdges = pnet.getInEdges(toBeRefined);
			sourceIter = sourceEdges.iterator();
		}
 
		while (targetIter.hasNext())
		{
			Arc arc = (Arc) targetIter.next();
			elementWeight.removeElementWeight(arc);
			pnet.removeEdge(arc);//targetIter.next());
			
			targetEdges = pnet.getOutEdges(toBeRefined);
			targetIter = targetEdges.iterator();
		}

		elementWeight.removeElementWeight(toBeRefined);
		pnet.removeNode(toBeRefined);
	}
}
