package org.processmining.plugins.testbed.util;

import java.util.HashMap;
import java.util.Map;

import org.processmining.models.graphbased.directed.petrinet.elements.Arc;
import org.processmining.models.graphbased.directed.petrinet.elements.Place;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;


/**
 * the class manages the weights of all the nodes/arcs in a net
 * 
 * @author Zheng Liu
 * @email z.liu3@tue.nl
 * @version 1.0
 * @date 10/June/2010
 *
 */
public class ElementWeight 
{	
	private Map<Place, Integer> placeWeight;
	private Map<Transition,Integer> transitionWeight;
	private Map<Arc, Integer> arcWeight;
	
	
	public ElementWeight()
	{	
		placeWeight = new HashMap<Place, Integer> (); 
		transitionWeight = new HashMap<Transition, Integer>();
		arcWeight = new HashMap<Arc, Integer>(); 
	}
	
	
	/**
	 * returns the weights of all the places 
	 * 
	 * @return totalWeight, total place weight 
	 */
	public int getPlaceTotalWeight()
	{
		int totalWeight = 0;
		
		for (Place p : placeWeight.keySet())
		{
			totalWeight += placeWeight.get(p);
		}
		
		return totalWeight;
	}
	
	
	/**
	 * returns the weights of all the transitions 
	 * 
	 * @return totalWeight, total transition weight
	 */
	public int getTransitionTotalWeight()
	{
		int totalWeight = 0;
		
		for (Transition t : transitionWeight.keySet())
		{
			totalWeight += transitionWeight.get(t);
		}
		
		return totalWeight;
	}
	
	
	/**
	 * returns the weight of all the arcs
	 *  
	 * @return totalWeight, total arc weight
	 */
	public int getArcTotalWeight()
	{
		int totalWeight = 0;
		
		for (Arc r : arcWeight.keySet())
		{
			totalWeight += arcWeight.get(r);
		}
		
		return totalWeight;
	}
	
	
	/**
	 * sets weight for a place
	 *  
	 * @param place, the place
	 * @param weight, the weight
	 */
	public void setWeight(Place place, int weight)
	{
		placeWeight.put(place, weight);
	}
	
	
	/**
	 * sets weight for a transition 
	 * 
	 * @param transition, the transition
	 * @param weight, the weight
	 */
	public void setWeight(Transition transition, int weight)
	{
		transitionWeight.put(transition, weight);
	}
	
	
	/**
	 * sets weight for an arc 
	 * 
	 * @param arc, the arc
	 * @param weight, the weight
	 */
	public void setWeight(Arc arc, int weight)
	{
		arcWeight.put(arc, weight);
	}
	
	
	
	//this is a stub
	/**
	 * computes weight based on one other weight 
	 * 
	 * @param weight
	 * 
	 * @return weight
	 */
	public int addElementWeight(int weight)
	{
		return weight;
	}
	
	
	
	//this is a stub
	/**
	 * computes weight based on two other weights
	 * 
	 * @param weight1
	 * @param weight2
	 * 
	 * @return weight 
	 */
	public int addElementWeight(int weight1, int weight2)
	{
		return (weight1 + weight2) /2;
	}
	
	
	
	/**
	 * returns all the places and their weights 
	 * 
	 * @return placeWeight, map of places and their weights 
	 */
	public Map<Place, Integer> getPlaceWeight()
	{
		return placeWeight;
	}
	
	
	
	/**
	 * returns all the transitions and their weights
	 * 
	 * @return transitionWeight, map of transitions and their weights 
	 */
	public Map<Transition, Integer> getTransitionWeight()
	{
		return transitionWeight;
	}
	
	
	
	/**
	 * returns all the arcs and their weights
	 *  
	 * @return arcWeight, map of arcs and their weights 
	 */
	public Map<Arc, Integer> getArcWeight()
	{
		return arcWeight;
	}
	
	
	
	/**
	 * removes the weight of a place
	 * 
	 * @param place, the place
	 */
	public void removeElementWeight(Place place)
	{
		placeWeight.remove(place);
	}
	
	
	
	/**
	 * removes the weight of a transition
	 * 
	 * @param transition, the transition
	 */
	public void removeElementWeight(Transition transition)
	{
		transitionWeight.remove(transition);
	}
	
	
	
	/**
	 * removes the weight of an arc
	 * 
	 * @param arc, the arc 
	 */
	public void removeElementWeight(Arc arc)
	{
		arcWeight.remove(arc);
	}

	
	
	
	/**
	 * returns the weight of a place 
	 * 
	 * @param place, the place 
	 * 
	 * @return weight, the weight of the place  
	 */
	public int getElementWeight(Place place)
	{
		return placeWeight.get(place);
	}
	
	
	
	/**
	 * returns the weight of a transition
	 * 
	 * @param transition, the transition
	 *  
	 * @return weight, the weight of the transition
	 */
	public int getElementWeight(Transition transition)
	{
		return transitionWeight.get(transition);
	}
	
	
	
	
	/**
	 * returns the weight of an arc 
	 * 
	 * @param arc, the arc
	 * 
	 * @return weight, the weight of the arc  
	 */
	public int getElementWeight(Arc arc)
	{
		return arcWeight.get(arc);
	}
	
	

	
	/**
	 * changes the weight of a place 
	 * 
	 * @param place, the place 
	 */
	public void changeElementWeight(Place place)
	{
		int weight = placeWeight.get(place);
		placeWeight.remove(place);
		placeWeight.put(place, weight);
	}
	
	
	
	
	
	/**
	 * changes the weight of a transition 
	 * 
	 * @param transition, the transition 
	 */
	public void changeElementWeight(Transition transition)
	{
		int weight = transitionWeight.get(transition);
		transitionWeight.remove(transition);
		transitionWeight.put(transition, weight); 
	}
	
	
	
	
	/**
	 * changes the weight of an arc
	 * 
	 * @param arc, the arc 
	 */
	public void changeElementWeight(Arc arc)
	{
		int weight = arcWeight.get(arc);
		arcWeight.remove(arc);
		arcWeight.put(arc, weight); 
	}
	
	
	
	
	
	/**
	 * gives a place a new given weight
	 *  
	 * @param place, the place
	 * @param weight, the new given weight to the place
	 */
	public void changeElementWeight(Place place, int weight)
	{
		placeWeight.remove(place);
		placeWeight.put(place, weight);
	}
	
	
	
	
	/**
	 * gives a transition a new given weight
	 * 
	 * @param transition, the transition 
	 * @param weight, the new given weight to the transition 
	 */
	public void changeElementWeight(Transition transition, int weight)
	{
		transitionWeight.remove(transition);
		transitionWeight.put(transition, weight);
	}
	
	
	
	/**
	 * gives an arc a new given weight
	 * 
	 * @param arc, the arc 
	 * @param weight, the new given weight to the arc 
	 */
	public void changeElementWeight(Arc arc, int weight)
	{
		arcWeight.remove(arc);
		arcWeight.put(arc, weight);
	}
}
